<?php
/*
Plugin Name: Doorman
Plugin URI: http://www.elektroelch.net
Description: BBounces registration attempts of spam bots and spammers with one time mailer addresses.
Version: 0.2
Author: Lutz Schr&ouml;er
Author URI: http://elektroelch.net/blog

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

if (! class_exists('Doorman')) {

	class Doorman {

		  private $options;
			private $servers;

			//------------------------------------------------------------------------------
			/**
			 * Initiate hooks, load textdomain
			 *
			 * @global string $pagenow Current page
			 *
			 * @since 0.1
			 * 
			 */
			function __construct() {
					global $pagenow;

					if ( is_admin() ) {
							register_activation_hook(__FILE__, array(&$this, 'activate'));
							add_action('admin_menu', array(&$this, 'admin_menu'));
							$plugin = plugin_basename(__FILE__);
							add_filter("plugin_action_links_$plugin", array(&$this, 'add_settings_link' ));
							add_action('admin_init', array(&$this, 'admin_init'));
					}

					load_plugin_textdomain('doorman', false, dirname(plugin_basename(__FILE__)) . '/lang');

				  // do actual checking
					if ( 'wp-login.php' == $pagenow ) {
							add_action('register_post', array(&$this, 'check_registration'), 10, 3);
					} //if
					
					
			} //__construct()
			//------------------------------------------------------------------------------
			/**
			 * Creates default settings on plugin activation
			 *
			 * @since 0.1
			 * 
			 */
			function activate(){

					// todo: only save options if not already present
					$server_data = file(plugin_dir_path( __FILE__ ) . 'servers.dat');
					$options = array();
					$servers = array();
					foreach ($server_data as $server) {
							$servers[] = trim(strtolower($server));
					}
					$options['error_message'] = __('The domain %s has been blocked by the system.', 'doorman');
					$options['servers'] = $servers;

					add_option('doorman', $options);

			} //mailbag_activate()

			/* -------------------------------------------------------------------------------------------------------------- */
			function admin_menu() {
				/**
				 * Adds option page to backend menu
				 *
				 * @since 0.1
				 * 
				 */
				$page = add_options_page('Doorman', 'Doorman', 'manage_options', 'doorman', array(&$this, 'plugin_options'));
			}
			/* -------------------------------------------------------------------------------------------------------------- */
			/**
			 * Loads settings page
			 *
			 * @since 0.1
			 * 
			 */
			function plugin_options() {
					require_once('doorman_options.php');
			}
			/* -------------------------------------------------------------------------------------------------------------- */
			/**
			 * Loads CSS styles if doorman settings page is loaded
			 *
			 * @since 0.1
			 * 
			 */
			function admin_init() {
				global $pagenow;
				if ( 'options-general.php' == $pagenow && isset($_GET['page']) && 'doorman' == $_GET['page'] ) {
					wp_enqueue_style('dashboard');
					wp_enqueue_style('thickbox');
					wp_enqueue_style('global');
					wp_enqueue_style('wp-admin');
					wp_enqueue_script( 'postbox' );
					wp_enqueue_script( 'dashboard' );
					wp_enqueue_script( 'thickbox' );
					add_action( 'admin_print_styles', array(&$this,'setup_styles'));
				} //if
			} //admin_init()
			/* -------------------------------------------------------------------------------------------------------------- */
			/**
			 * Loads CSS styles specific to doorman
			 * 
			 * @since 0.1
			 * 
			 */
			function setup_styles() {
					wp_enqueue_style('doorman_styles', dirname(plugins_url(__FILE__)) . '/doorman.css');
			}
			/* -------------------------------------------------------------------------------------------------------------- */
			/**
			 * Adds link to doorman settings to plugin page
			 *
			 * @since 0.1
			 * 
			 * @param unknown $links Description
			 * 
			 * @return Type    Description	
			 */
			function add_settings_link( $links ) {
					$settings_link = '<a href="options-general.php?page=doorman.php">'. __('Settings').'</a>';
					array_unshift( $links, $settings_link );
					return $links;
			} //add_settings_link()
	
			/* -------------------------------------------------------------------------------------------------------------- */
			/**
			 * Checks mail domain entered by user against internal list
			 * of disposable mail services
			 *
			 * @since 0.1
			 *
			 * @param $domain Domain name entered in the registration form
			 *
			 * @return bool TRUE, if server is in internal list
			 */
			function check_internal_list($domain) {
				return (sizeof(preg_grep('/'. $domain . '$/', $this->options['servers'])) > 0);
			}

			/* -------------------------------------------------------------------------------------------------------------- */
			/**
			 * Checks mail domain entered by user against blacklist
			 *
			 * @since 0.1
			 *
			 * @param $domain Domain name or part of it as defined in the blacklist
			 *
			 * @return bool TRUE, if server is in blacklist
			 */
			function check_blacklist($domain) {
				$banned = isset($this->options['blacklist']) ? implode('|', $this->options['blacklist']) : NULL;
				if ($banned) 
				  return (preg_match('/('. $banned . ')$/', $domain) > 0);
				return false;
			}

			/* -------------------------------------------------------------------------------------------------------------- */
			/**
			 * Checks mail domain entered by user against whitelist
			 *
			 * @param $domain Domain name or part of it as defined in the whitelist
			 *
			 * @return bool TRUE, if server is in whitelist
 			 *
			 * @since 0.1
			 * 
			 */
			function check_whitelist($domain) {
				$allowed = isset($this->options['whitelist']) ? implode('|', $this->options['whitelist']) : NULL;
				if ($allowed)
					return (preg_match('/('. $allowed . ')$/', $domain) > 0);
				return false;
			}
			/* -------------------------------------------------------------------------------------------------------------- */
			/**
			 * Check user email with botscout.com
			 * 
			 * @param string $user_email User email given at login form
			 * 
			 * @return boolean    TRUE if found at botscout.com, otherwise false
			 *
			 * @since 0.2
			 * 
			 */
			function check_botscout($user_email) {
				require_once('botscout.php');
				$bs = new DoormanBotscout;
				if (isset($this->options['botscoutapikey']))
					$bs->set_apikey($this->options['botscoutapikey']);
				$bs->set_xmail($user_email);
				$response = $bs->check_mail();

				// if $response is not set there has been an error
				// assume mail address to be valid
				if (! isset($response['response']))
					return false;
				
				$data = explode('|', $response['body']);
				
				return ('Y' == $data[0]);

			} //
			/* -------------------------------------------------------------------------------------------------------------- */
			/**
			 *
			 * Calls various checking methods and sets error message accordingly
			 *
			 * @since 0.1
			 * 
			 * @param $sanitized_user_login
			 * @param $user_email
			 * @param $errors
			 *
			 */
			function check_registration( $sanitized_user_login, $user_email, $errors ) {

					$this->options = get_option( 'doorman' );

					$user_email = strtolower( $user_email );
					list($name, $domain) = explode('@', $user_email);

					// if domain is in white list the user can be registered
					if ($this->check_whitelist($domain))
							return;

					// do all checks, stops if a check returns true
					$found = $this->check_blacklist($domain)
									 || $this->check_internal_list($domain)
									 || $this->check_botscout($user_email);

					if ($found) {
							$errors->add( 'registered', __( '<strong>ERROR</strong>: ' . sprintf($this->options['error_message'], $domain), 'message' ) );
							$user_email = '';
					} //if

					return;
			} //check_registration()

	} //Doorman
	
} //if

$doorman = new Doorman;

?>