<?php

// make sure we're in a WodPress environment
if ( ! function_exists( 'add_filter' ) ) {
	echo "Hi there! I'm just a part of plugin, not much I can do when called directly.";
	exit;
}

class DoormanOptions {

    /* -------------------------------------------------------------------------------------------------------------- */
		/**
		 * Save options to database. Loads "servers.dat" from plugin directory
		 *
		 * @since 0.1
		 * 
		 */
    function save_options() {


        $server_data = file(plugin_dir_path( __FILE__ ) . 'servers.dat');
        $options = array();
        $servers = array();
        foreach ($server_data as $server)
            $servers[] = trim(strtolower($server));

        if (isset($_POST['whitelist']))
            $options['whitelist'] = array_filter(preg_split( "/(\s)/", $_POST['whitelist']));
        if (isset($_POST['blacklist']))
            $options['blacklist'] = array_filter(preg_split( "/(\s)/", $_POST['blacklist']));
        if (isset($_POST['error_message']))
            $options['error_message'] = $_POST['error_message'];
        if (isset($_POST['botscoutapikey']))
            $options['botscoutapikey'] = $_POST['botscoutapikey'];
			
        $options['servers'] = $servers;

        update_option('doorman', $options);

    }

    /* -------------------------------------------------------------------------------------------------------------- */
		/**
		 * Loads options and display form
		 *
		 * @since 0.1
		 * 
		 */
    function __construct() {
        if ( ! current_user_can('manage_options') )  {
            wp_die( __('You do not have sufficient permissions to access this page.') );
        }
        $this->options = get_option('doorman');
        $this->form();

    }

    /* -------------------------------------------------------------------------------------------------------------- */
		/**
		 * Create postbox
		 * 
		 * @param string $id      id of postbox
		 * @param string $title   title of postbox
		 * @param string $content content of postbox
		 *
		 * @since 0.1
		 * 
		 */
    function postbox($id, $title, $content) {
    	?>
    		<div id="<?php echo $id; ?>" class="postbox">
    			<div class="handlediv" title="<?php __('Click to toggle', 'doorman'); ?>"><br /></div>
    			<h3 class="hndle"><span><?php echo $title; ?></span></h3>
    			<div class="inside">
    				<?php echo $content; ?>
    			</div>
    		</div>
    	<?php
    }
    /* -------------------------------------------------------------------------------------------------------------- */
		/**
		 * Displays form
		 * 
		 * @since 0.1
		 * 
		 */
    function form() {
        if (isset($_POST['doorman-form'])) {
            $this->save_options();
            echo '<div id="message" class="updated fade"><p><strong>' . __('Settings saved.') . '</strong>   </p></div>';
        }
        ?>
        <div class="wrap">
        <h2>
            <?php esc_html_e('Doorman', 'doorman'); ?>
        </h2>
        <?php
        $this->options = get_option('doorman');
        ?>
        <form method="post" action="<?php echo $_SERVER ['REQUEST_URI']; ?>">
        <?php wp_nonce_field( 'doorman_update' ); ?>
        <input type="hidden" name="doorman-form" value="1" />

        <div class="postbox-container" style="width:70%;">
			<div class="metabox-holder">
				<div class="meta-box-sortables">
				<?php
				
						$content = '<input type="text" id="error_message" name="error_message" style="width:100%" value="';
						$content .= isset($this->options['error_message']) ? $this->options['error_message'] : '';
						$content .= '">';
						$this->postbox('error_message', __('Error message', 'doorman'), $content);

						$content = '<textarea style="height: 12em; width: 100%;" name="whitelist" id="whitelist" size="300">';
								if (isset($this->options['whitelist']))
										foreach ($this->options['whitelist'] as $whitelisted)
												$content .= $whitelisted . "\n";
						$content .= '</textarea>';
						$this->postbox('whitelist', __('Whitelist', 'doorman'), $content);
				
						$content = '<textarea style="height: 12em; width: 100%;" name="blacklist" id="blacklist" size="300">';
						if (isset($this->options['blacklist']))
								foreach ($this->options['blacklist'] as $blacklisted)
										$content .= $blacklisted . "\n";
						$content .= '</textarea>';
						$this->postbox('blacklist', __('Blacklist', 'doorman'), $content);
						

						$content = __('API key:', 'doorman');
						$content .= '<input type="text" name="botscoutapikey" size="50"';
						if (isset($this->options['botscoutapikey']))
						  $content .= '" value="' . $this->options['botscoutapikey'] . '"';
						$content .= '>';
						$content .= __("<p>Currently you can do 20 bot lookup tests per day. If you need more than that you can get an API key (it's free). ", 'doorman');
						$content .= sprintf( __('You can get the key here: %s</p>', 'doorman'), 'http://botscout.com/getkey.htm');
						$this->postbox('botscout', __('Botscout', 'doorman'), $content);


						$servers = count($this->options['servers']);
						$content = __(sprintf(__('There are %s servers in the list.', 'doorman'), $servers));
						//todo: load serverlist viaajax call
						//$content .= '<a jref="#">' . __('Display.') . '</a>';
						$this->postbox('server_list', __('Server list', 'doorman'), $content);

        ?>
                </div>
            <div class="submit">
                <input type="submit" class="button-primary" name="submit" value="<?php _e("Save Settings", 'doorman'); ?>" />
            </div>

        </div>
        </div>
        </form>

    </div>
        <?php
    }
} //class

$doorman_options = new DoormanOptions;

?>